@@ -32,6 +32,7 @@ enum swErrorCode
     SW_ERROR_FILE_TOO_LARGE,
     SW_ERROR_FILE_EMPTY,
     SW_ERROR_DNSLOOKUP_DUPLICATE_REQUEST,
+    SW_ERROR_DNSLOOKUP_RESOLVE_FAILED,
 
     /**
      * connection error
@@ -75,6 +76,9 @@ enum swErrorCode
     SW_ERROR_HTTP_INVALID_PROTOCOL,
 
     SW_ERROR_WEBSOCKET_BAD_CLIENT = 8501,
+    SW_ERROR_WEBSOCKET_BAD_OPCODE,
+    SW_ERROR_WEBSOCKET_UNCONNECTED,
+    SW_ERROR_WEBSOCKET_HANDSHAKE_FAILED,
 
     /**
      * server global error
