@@ -42,7 +42,6 @@ static int swClient_onWrite(swReactor *reactor, swEvent *event);
 static int swClient_onError(swReactor *reactor, swEvent *event);
 static void swClient_onTimeout(swTimer *timer, swTimer_node *tnode);
 static void swClient_onResolveCompleted(swAio_event *event);
-static int swClient_onPackage(swConnection *conn, char *data, uint32_t length);
 
 static sw_inline void execute_onConnect(swClient *cli)
 {
@@ -103,15 +102,7 @@ int swClient_create(swClient *cli, int type, int async)
 
     if (async)
     {
-        if (swIsMaster() && SwooleTG.type == SW_THREAD_REACTOR)
-        {
-            cli->reactor = SwooleTG.reactor;
-        }
-        else
-        {
-            cli->reactor = SwooleG.main_reactor;
-        }
-        cli->socket = swReactor_get(cli->reactor, sockfd);
+        cli->socket = swReactor_get(SwooleG.main_reactor, sockfd);
     }
     else
     {
@@ -134,12 +125,12 @@ int swClient_create(swClient *cli, int type, int async)
     if (async)
     {
         swSetNonBlock(cli->socket->fd);
-        if (!swReactor_handle_isset(cli->reactor, SW_FD_STREAM_CLIENT))
+        if (!swReactor_handle_isset(SwooleG.main_reactor, SW_FD_STREAM_CLIENT))
         {
-            cli->reactor->setHandle(cli->reactor, SW_FD_STREAM_CLIENT | SW_EVENT_READ, swClient_onStreamRead);
-            cli->reactor->setHandle(cli->reactor, SW_FD_DGRAM_CLIENT | SW_EVENT_READ, swClient_onDgramRead);
-            cli->reactor->setHandle(cli->reactor, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE, swClient_onWrite);
-            cli->reactor->setHandle(cli->reactor, SW_FD_STREAM_CLIENT | SW_EVENT_ERROR, swClient_onError);
+            SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_STREAM_CLIENT | SW_EVENT_READ, swClient_onStreamRead);
+            SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_DGRAM_CLIENT | SW_EVENT_READ, swClient_onDgramRead);
+            SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE, swClient_onWrite);
+            SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_STREAM_CLIENT | SW_EVENT_ERROR, swClient_onError);
         }
     }
 
@@ -159,14 +150,12 @@ int swClient_create(swClient *cli, int type, int async)
             cli->send = swClient_tcp_send_sync;
             cli->sendfile = swClient_tcp_sendfile_sync;
         }
-        cli->reactor_fdtype = SW_FD_STREAM_CLIENT;
     }
     else
     {
         cli->connect = swClient_udp_connect;
         cli->recv = swClient_udp_recv;
         cli->send = swClient_udp_send;
-        cli->reactor_fdtype = SW_FD_DGRAM_CLIENT;
     }
 
     cli->_sock_domain = _domain;
@@ -180,7 +169,6 @@ int swClient_create(swClient *cli, int type, int async)
     cli->protocol.package_length_size = 4;
     cli->protocol.package_body_offset = 0;
     cli->protocol.package_max_length = SW_BUFFER_INPUT_SIZE;
-    cli->protocol.onPackage = swClient_onPackage;
 
     return SW_OK;
 }
@@ -190,11 +178,11 @@ int swClient_sleep(swClient *cli)
     int ret;
     if (cli->socket->events & SW_EVENT_WRITE)
     {
-        ret = cli->reactor->set(cli->reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_WRITE);
+        ret = SwooleG.main_reactor->set(SwooleG.main_reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_WRITE);
     }
     else
     {
-        ret = cli->reactor->del(cli->reactor, cli->socket->fd);
+        ret = SwooleG.main_reactor->del(SwooleG.main_reactor, cli->socket->fd);
     }
     if (ret)
     {
@@ -208,11 +196,11 @@ int swClient_wakeup(swClient *cli)
     int ret;
     if (cli->socket->events & SW_EVENT_WRITE)
     {
-        ret = cli->reactor->set(cli->reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ | SW_EVENT_WRITE);
+        ret = SwooleG.main_reactor->set(SwooleG.main_reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ | SW_EVENT_WRITE);
     }
     else
     {
-        ret = cli->reactor->add(cli->reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ);
+        ret = SwooleG.main_reactor->add(SwooleG.main_reactor, cli->socket->fd, cli->socket->fdtype | SW_EVENT_READ);
     }
     if (ret)
     {
@@ -221,54 +209,6 @@ int swClient_wakeup(swClient *cli)
     return ret;
 }
 
-int swClient_shutdown(swClient *cli, int __how)
-{
-    if (!cli->socket || cli->socket->closed)
-    {
-        return SW_ERR;
-    }
-    if (__how == SHUT_RD)
-    {
-        if (cli->shutdown_read || cli->shutdow_rw || shutdown(cli->socket->fd, SHUT_RD))
-        {
-            return SW_ERR;
-        }
-        else
-        {
-            cli->shutdown_read = 1;
-            return SW_OK;
-        }
-    }
-    else if (__how == SHUT_WR)
-    {
-        if (cli->shutdown_write || cli->shutdow_rw || shutdown(cli->socket->fd, SHUT_RD) < 0)
-        {
-            return SW_ERR;
-        }
-        else
-        {
-            cli->shutdown_write = 1;
-            return SW_OK;
-        }
-    }
-    else if (__how == SHUT_RDWR)
-    {
-        if (cli->shutdow_rw || shutdown(cli->socket->fd, SHUT_RDWR) < 0)
-        {
-            return SW_ERR;
-        }
-        else
-        {
-            cli->shutdown_read = 1;
-            return SW_OK;
-        }
-    }
-    else
-    {
-        return SW_ERR;
-    }
-}
-
 #ifdef SW_USE_OPENSSL
 int swClient_enable_ssl_encrypt(swClient *cli)
 {
@@ -410,7 +350,6 @@ static int swClient_inet_addr(swClient *cli, char *host, int port)
     {
         if (swoole_gethostbyname(cli->_sock_domain, host, s_addr) < 0)
         {
-            SwooleG.error = SW_ERROR_DNSLOOKUP_RESOLVE_FAILED;
             return SW_ERR;
         }
     }
@@ -454,7 +393,6 @@ static int swClient_close(swClient *cli)
 {
     int fd = cli->socket->fd;
     assert(fd != 0);
-
 #ifdef SW_USE_OPENSSL
     if (cli->open_ssl && cli->ssl_context)
     {
@@ -509,9 +447,9 @@ static int swClient_close(swClient *cli)
     if (cli->async)
     {
         //remove from reactor
-        if (!cli->socket->removed && cli->reactor)
+        if (!cli->socket->removed && SwooleG.main_reactor)
         {
-            cli->reactor->del(cli->reactor, fd);
+            SwooleG.main_reactor->del(SwooleG.main_reactor, fd);
         }
         if (cli->timer)
         {
@@ -659,27 +597,16 @@ static int swClient_tcp_connect_async(swClient *cli, char *host, int port, doubl
 
     if (cli->wait_dns)
     {
-        if (SwooleAIO.mode == SW_AIO_LINUX)
-        {
-            SwooleAIO.mode = SW_AIO_BASE;
-            SwooleAIO.init = 0;
-        }
-        if (SwooleAIO.init == 0)
-        {
-            swAio_init();
-        }
-
         swAio_event ev;
         bzero(&ev, sizeof(swAio_event));
 
-        int len = strlen(cli->server_host);
         if (strlen(cli->server_host) < SW_IP_MAX_LENGTH)
         {
             ev.nbytes = SW_IP_MAX_LENGTH;
         }
         else
         {
-            ev.nbytes = len + 1;
+            ev.nbytes = strlen(host) + 1;
         }
 
         ev.buf = sw_malloc(ev.nbytes);
@@ -689,22 +616,14 @@ static int swClient_tcp_connect_async(swClient *cli, char *host, int port, doubl
             return SW_ERR;
         }
 
-        memcpy(ev.buf, cli->server_host, len);
-        ((char *) ev.buf)[len] = 0;
+        memcpy(ev.buf, cli->server_host, strlen(cli->server_host));
+
         ev.flags = cli->_sock_domain;
         ev.type = SW_AIO_DNS_LOOKUP;
         ev.object = cli;
         ev.callback = swClient_onResolveCompleted;
 
-        if (swAio_dispatch(&ev) < 0)
-        {
-            sw_free(ev.buf);
-            return SW_ERR;
-        }
-        else
-        {
-            return SW_OK;
-        }
+        return swAio_dispatch(&ev);
     }
 
     while (1)
@@ -723,7 +642,7 @@ static int swClient_tcp_connect_async(swClient *cli, char *host, int port, doubl
 
     if ((ret < 0 && errno == EINPROGRESS) || ret == 0)
     {
-        if (cli->reactor->add(cli->reactor, cli->socket->fd, cli->reactor_fdtype | SW_EVENT_WRITE) < 0)
+        if (SwooleG.main_reactor->add(SwooleG.main_reactor, cli->socket->fd, cli->reactor_fdtype | SW_EVENT_WRITE) < 0)
         {
             return SW_ERR;
         }
@@ -779,7 +698,7 @@ static int swClient_tcp_pipe(swClient *cli, int write_fd, int flags)
 static int swClient_tcp_send_async(swClient *cli, char *data, int length, int flags)
 {
     int n = length;
-    if (cli->reactor->write(cli->reactor, cli->socket->fd, data, length) < 0)
+    if (SwooleG.main_reactor->write(SwooleG.main_reactor, cli->socket->fd, data, length) < 0)
     {
         if (SwooleG.error == SW_ERROR_OUTPUT_BUFFER_OVERFLOW)
         {
@@ -854,12 +773,12 @@ static int swClient_tcp_sendfile_async(swClient *cli, char *filename, off_t offs
     {
         if (cli->socket->events & SW_EVENT_READ)
         {
-            return cli->reactor->set(cli->reactor, cli->socket->fd,
+            return SwooleG.main_reactor->set(SwooleG.main_reactor, cli->socket->fd,
                     cli->socket->fdtype | SW_EVENT_READ | SW_EVENT_WRITE);
         }
         else
         {
-            return cli->reactor->add(cli->reactor, cli->socket->fd,
+            return SwooleG.main_reactor->add(SwooleG.main_reactor, cli->socket->fd,
                     cli->socket->fdtype | SW_EVENT_WRITE);
         }
     }
@@ -882,18 +801,7 @@ static int swClient_tcp_recv_no_buffer(swClient *cli, char *data, int len, int f
         }
         if (errno == EINTR)
         {
-            if (cli->interrupt_time <= 0)
-            {
-                cli->interrupt_time = swoole_microtime();
-            }
-            else if (swoole_microtime() > cli->interrupt_time + cli->timeout)
-            {
-                break;
-            }
-            else
-            {
-                continue;
-            }
+            continue;
         }
 #ifdef SW_USE_OPENSSL
         if (errno == EAGAIN && cli->socket->ssl)
@@ -944,7 +852,7 @@ static int swClient_udp_connect(swClient *cli, char *host, int port, double time
             return SW_ERR;
         }
     }
-    if (udp_connect != 1)
+    else if (udp_connect != 1)
     {
         goto connect_ok;
     }
@@ -959,7 +867,7 @@ static int swClient_udp_connect(swClient *cli, char *host, int port, double time
 
         if (cli->async && cli->onConnect)
         {
-            if (cli->reactor->add(cli->reactor, cli->socket->fd, cli->reactor_fdtype | SW_EVENT_READ) < 0)
+            if (SwooleG.main_reactor->add(SwooleG.main_reactor, cli->socket->fd, cli->reactor_fdtype | SW_EVENT_READ) < 0)
             {
                 return SW_ERR;
             }
@@ -971,7 +879,6 @@ static int swClient_udp_connect(swClient *cli, char *host, int port, double time
     {
         swSysError("connect() failed.");
         cli->socket->active = 0;
-        cli->socket->removed = 1;
         return SW_ERR;
     }
 }
@@ -1071,13 +978,6 @@ static int swClient_https_proxy_handshake(swClient *cli)
 }
 #endif
 
-static int swClient_onPackage(swConnection *conn, char *data, uint32_t length)
-{
-    swClient *cli = (swClient *) conn->object;
-    cli->onReceive(conn->object, data, length);
-    return conn->close_wait ? SW_ERR : SW_OK;
-}
-
 static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
 {
     int n;
@@ -1124,7 +1024,7 @@ static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
                 {
                     cli->socket->ssl_state = SW_SSL_STATE_WAIT_STREAM;
                 }
-                return cli->reactor->set(cli->reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE);
+                return SwooleG.main_reactor->set(SwooleG.main_reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE);
             }
             if (cli->onConnect)
             {
@@ -1172,7 +1072,7 @@ static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
                 {
                     cli->socket->ssl_state = SW_SSL_STATE_WAIT_STREAM;
                 }
-                return cli->reactor->set(cli->reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE);
+                return SwooleG.main_reactor->set(SwooleG.main_reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_WRITE);
             }
         }
         else
@@ -1191,7 +1091,7 @@ static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
     {
         if (swClient_ssl_handshake(cli) < 0)
         {
-            goto connect_fail;
+            return cli->close(cli);
         }
         if (cli->socket->ssl_state != SW_SSL_STATE_READY)
         {
@@ -1222,7 +1122,7 @@ static int swClient_onStreamRead(swReactor *reactor, swEvent *event)
         }
         if (cli->_redirect_to_socket)
         {
-            ret = cli->reactor->write(cli->reactor, cli->_redirect_to_socket, buf, n);
+            ret = SwooleG.main_reactor->write(SwooleG.main_reactor, cli->_redirect_to_socket, buf, n);
         }
         else if (cli->_redirect_to_session)
         {
@@ -1375,7 +1275,7 @@ static void swClient_onResolveCompleted(swAio_event *event)
     swClient *cli = event->object;
     cli->wait_dns = 0;
 
-    if (event->error == 0)
+    if (event->ret == 0)
     {
         swClient_tcp_connect_async(cli, event->buf, cli->server_port, cli->timeout, 1);
     }
@@ -1414,13 +1314,13 @@ static int swClient_onWrite(swReactor *reactor, swEvent *event)
             {
                 if (_socket->ssl_want_read)
                 {
-                    cli->reactor->set(cli->reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_READ);
+                    SwooleG.main_reactor->set(SwooleG.main_reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_READ);
                 }
                 return SW_OK;
             }
         }
 #endif
-        if (swReactor_onWrite(cli->reactor, event) < 0)
+        if (swReactor_onWrite(SwooleG.main_reactor, event) < 0)
         {
             return SW_ERR;
         }
@@ -1443,7 +1343,7 @@ static int swClient_onWrite(swReactor *reactor, swEvent *event)
     if (SwooleG.error == 0)
     {
         //listen read event
-        cli->reactor->set(cli->reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_READ);
+        SwooleG.main_reactor->set(SwooleG.main_reactor, event->fd, SW_FD_STREAM_CLIENT | SW_EVENT_READ);
         //connected
         _socket->active = 1;
         //socks5 proxy
