@@ -190,6 +190,7 @@ int swAioBase_init(int max_aio_events)
         return SW_ERR;
     }
 
+    SwooleAIO.callback = swAio_callback_test;
     SwooleAIO.destroy = swAioBase_destroy;
     SwooleAIO.read = swAioBase_read;
     SwooleAIO.write = swAioBase_write;
@@ -246,13 +247,7 @@ static int swAioBase_thread_onTask(swThreadPool *pool, void *task, int task_len)
         }
         break;
     case SW_AIO_DNS_LOOKUP:
-#ifndef HAVE_GETHOSTBYNAME2_R
-        SwooleAIO.lock.lock(&SwooleAIO.lock);
-#endif
         ret = swoole_gethostbyname(event->flags == AF_INET6 ? AF_INET6 : AF_INET, event->buf, (char *) &addr);
-#ifndef HAVE_GETHOSTBYNAME2_R
-        SwooleAIO.lock.unlock(&SwooleAIO.lock);
-#endif
         if (ret < 0)
         {
             event->error = h_errno;
@@ -275,11 +270,6 @@ static int swAioBase_thread_onTask(swThreadPool *pool, void *task, int task_len)
             ret = 0;
         }
         break;
-
-    case SW_AIO_GETADDRINFO:
-        event->error = swoole_getaddrinfo((swRequest_getaddrinfo *) event->req);
-        break;
-
     default:
         swWarn("unknow aio task.");
         break;
