@@ -673,7 +673,7 @@ int swSSL_accept(swConnection *conn)
     {
         return SW_ERROR;
     }
-    swWarn("SSL_do_handshake() failed. Error: [%ld|%d].", err, errno);
+    swWarn("SSL_do_handshake() failed. Error: [%ld].", err);
     return SW_ERROR;
 }
 
@@ -720,16 +720,12 @@ int swSSL_sendfile(swConnection *conn, int fd, off_t *offset, size_t size)
         ret = swSSL_send(conn, buf, n);
         if (ret < 0)
         {
-            if (swConnection_error(errno) == SW_ERROR)
-            {
-                swSysError("write() failed.");
-            }
+            swSysError("write() failed.");
         }
         else
         {
             *offset += ret;
         }
-        swTraceLog(SW_TRACE_REACTOR, "fd=%d, readn=%d, n=%d, ret=%d", fd, readn, n, ret);
         return ret;
     }
     else
