@@ -27,10 +27,13 @@
 #endif
 #endif
 
+#ifdef SW_COROUTINE
+#include "coroutine.h"
+#endif
+
 static void swReactor_onTimeout_and_Finish(swReactor *reactor);
 static void swReactor_onTimeout(swReactor *reactor);
 static void swReactor_onFinish(swReactor *reactor);
-static void swReactor_onBegin(swReactor *reactor);
 static int swReactor_defer(swReactor *reactor, swCallback callback, void *data);
 
 int swReactor_create(swReactor *reactor, int max_event)
@@ -114,28 +117,6 @@ static int swReactor_defer(swReactor *reactor, swCallback callback, void *data)
     return SW_OK;
 }
 
-int swReactor_empty(swReactor *reactor)
-{
-    //timer
-    if (SwooleG.timer.num > 0)
-    {
-        return SW_FALSE;
-    }
-
-    int empty = SW_FALSE;
-    //thread pool
-    if (SwooleAIO.init && reactor->event_num == 1 && SwooleAIO.task_num == 0)
-    {
-        empty = SW_TRUE;
-    }
-    //no event
-    else if (reactor->event_num == 0)
-    {
-        empty = SW_TRUE;
-    }
-    return empty;
-}
-
 /**
  * execute when reactor timeout and reactor finish
  */
@@ -146,23 +127,29 @@ static void swReactor_onTimeout_and_Finish(swReactor *reactor)
     {
         swTimer_select(&SwooleG.timer);
     }
-    //defer callback
-    swDefer_callback *cb, *tmp;
-    swDefer_callback *defer_callback_list = reactor->defer_callback_list;
-    reactor->defer_callback_list = NULL;
-    LL_FOREACH(defer_callback_list, cb)
-    {
-        cb->callback(cb->data);
-    }
-    LL_FOREACH_SAFE(defer_callback_list, cb, tmp)
-    {
-        sw_free(cb);
-    }
     //callback at the end
     if (reactor->idle_task.callback)
     {
         reactor->idle_task.callback(reactor->idle_task.data);
     }
+#ifdef SW_COROUTINE
+    //coro timeout
+    if (!swIsMaster())
+    {
+        coro_handle_timeout();
+    }
+#endif
+
+    //server master
+    if (SwooleG.serv && SwooleTG.update_time)
+    {
+        swoole_update_time();
+        int32_t timeout_msec = SwooleG.main_reactor->timeout_msec;
+        if (timeout_msec < 0 || timeout_msec > 1000)
+        {
+            SwooleG.main_reactor->timeout_msec = 1000;
+        }
+    }
     //server worker
     swWorker *worker = SwooleWG.worker;
     if (worker != NULL)
@@ -172,10 +159,17 @@ static void swReactor_onTimeout_and_Finish(swReactor *reactor)
             swWorker_try_to_exit();
         }
     }
-    //not server, the event loop is empty
-    if (SwooleG.serv == NULL  && swReactor_empty(reactor))
+    //client
+    if (SwooleG.serv == NULL && SwooleG.timer.num <= 0)
     {
-        reactor->running = 0;
+        if (SwooleAIO.init && reactor->event_num == 1 && SwooleAIO.task_num == 0)
+        {
+            reactor->running = 0;
+        }
+        else if (reactor->event_num == 0)
+        {
+            reactor->running = 0;
+        }
     }
 
 #ifdef SW_USE_MALLOC_TRIM
@@ -206,20 +200,18 @@ static void swReactor_onFinish(swReactor *reactor)
         swSignal_callback(reactor->singal_no);
         reactor->singal_no = 0;
     }
-    swReactor_onTimeout_and_Finish(reactor);
-}
-
-void swReactor_activate_future_task(swReactor *reactor)
-{
-    reactor->onBegin = swReactor_onBegin;
-}
-
-static void swReactor_onBegin(swReactor *reactor)
-{
-    if (reactor->future_task.callback)
+    //defer callback
+    swDefer_callback *cb, *tmp;
+    LL_FOREACH(reactor->defer_callback_list, cb)
     {
-        reactor->future_task.callback(reactor->future_task.data);
+        cb->callback(cb->data);
     }
+    LL_FOREACH_SAFE(reactor->defer_callback_list, cb, tmp)
+    {
+        sw_free(cb);
+    }
+    reactor->defer_callback_list = NULL;
+    swReactor_onTimeout_and_Finish(reactor);
 }
 
 int swReactor_close(swReactor *reactor, int fd)
@@ -259,12 +251,6 @@ int swReactor_write(swReactor *reactor, int fd, void *buf, int n)
         socket->buffer_size = SwooleG.socket_buffer_size;
     }
 
-    if (socket->nonblock == 0)
-    {
-        swoole_fcntl_set_option(fd, 1, -1);
-        socket->nonblock = 1;
-    }
-
     if (n > socket->buffer_size)
     {
         swoole_error_log(SW_LOG_WARNING, SW_ERROR_PACKAGE_LENGTH_TOO_LARGE, "data is too large, cannot exceed buffer size.");
