@@ -24,7 +24,6 @@
 #endif
 
 #define SW_MAX_FDTYPE              32   //32 kinds of event
-#define SW_MAX_HOOK_TYPE           32
 #define SW_ERROR_MSG_SIZE          512
 #define SW_MAX_WORKER_GROUP        2
 #define SW_MAX_FILE_CONTENT        (64*1024*1024) //for swoole_file_get_contents
@@ -61,6 +60,7 @@
 #define SW_MAX_WORKER_NCPU         1000 // n * cpu_num
 #define SW_MAX_REQUEST             5000          //最大请求包数
 
+#define SW_CORO_SCHEDUER_TIMEOUT   100           //协程强制超时回调的单位时间 100ms
 //#define SW_CONNECTION_LIST_EXPAND  (4096*2)  //动态扩容的数量
 
 #define SW_HOST_MAXSIZE            104  // Linux has 108 UNIX_PATH_MAX, but BSD/MacOS limit is only 104
@@ -68,7 +68,7 @@
 //#define SW_DEBUG                 //debug
 #define SW_LOG_NO_SRCINFO          //no source info
 #define SW_LOG_TRACE_OPEN          0
-#define SW_LOG_TRACE_FLAGS         (SW_TRACE_EVENT | SW_TRACE_REACTOR | SW_TRACE_CLOSE)
+#define SW_LOG_TRACE_FLAGS         (SW_TRACE_REDIS_CLIENT)
 //#define SW_BUFFER_SIZE           65495 //65535 - 28 - 12(UDP最大包 - 包头 - 3个INT)
 #define SW_CLIENT_BUFFER_SIZE      65536
 //#define SW_CLIENT_RECV_AGAIN
@@ -106,8 +106,14 @@
 #define SW_HEARTBEAT_PING_LEN      8
 #define SW_HEARTBEAT_PONG_LEN      8
 
+#define SW_MAINREACTOR_USE_UNSOCK  1    //主线程使用unsock
+#define SW_REACTOR_WRITER_TIMEO    3    //writer线程的reactor
 #define SW_TASKWAIT_TIMEOUT        0.5
 
+#ifdef HAVE_EVENTFD
+#define HAVE_LINUX_AIO
+#endif
+
 #define SW_AIO_THREAD_NUM_DEFAULT        2
 #define SW_AIO_THREAD_NUM_MAX            32
 #define SW_AIO_MAX_FILESIZE              4194304  //4M
@@ -276,6 +282,6 @@
 
 #define SW_REDIS_CONNECT_TIMEOUT         1.0
 
-#define SW_TIMER_MAX_VALUE               86400000
+#define SW_PHP_FUNCTION_MAX_ARG          16
 
 #endif /* SWOOLE_CONFIG_H_ */
