@@ -16,105 +16,10 @@
 */
 
 #include "php_swoole.h"
-#include "websocket.h"
-#include "thirdparty/php_http_parser.h"
-
-#include "ext/standard/base64.h"
-
-#ifdef SW_HAVE_ZLIB
-#include <zlib.h>
-#endif
-
-enum http_client_state
-{
-    HTTP_CLIENT_STATE_WAIT,
-    HTTP_CLIENT_STATE_READY,
-    HTTP_CLIENT_STATE_BUSY,
-    //WebSocket
-    HTTP_CLIENT_STATE_UPGRADE,
-    HTTP_CLIENT_STATE_WAIT_CLOSE,
-};
-
-typedef struct
-{
-    zval *onConnect;
-    zval *onError;
-    zval *onClose;
-    zval *onMessage;
-    zval *onResponse;
-
-#if PHP_MAJOR_VERSION >= 7
-    zval _object;
-    zval _request_upload_files;
-    zval _download_file;
-    zval _onConnect;
-    zval _onError;
-    zval _onClose;
-    zval _onMessage;
-#endif
-    zval *request_upload_files;
-    zval *download_file;
-    off_t download_offset;
-    char *request_method;
-    int callback_index;
-    uint8_t shutdown;
-    uint8_t request_timeout;
-
-} http_client_property;
-
-typedef struct
-{
-    swClient *cli;
-    char *host;
-    zend_size_t host_len;
-    long port;
-    double timeout;
-    char* uri;
-    zend_size_t uri_len;
-
-    swTimer_node *timer;
-
-#ifdef SW_HAVE_ZLIB
-    z_stream gzip_stream;
-    swString *gzip_buffer;
-#endif
-
-    /**
-     * download page
-     */
-    int file_fd;
-
-    php_http_parser parser;
-
-    char *tmp_header_field_name;
-    zend_size_t tmp_header_field_name_len;
-    swString *body;
-
-    uint8_t state;       //0 wait 1 ready 2 busy
-    uint8_t keep_alive;  //0 no 1 keep
-    uint8_t upgrade;
-    uint8_t gzip;
-    uint8_t chunked;     //Transfer-Encoding: chunked
-    uint8_t completed;
-    uint8_t websocket_mask;
-    uint8_t download;    //save http response to file
-    uint8_t header_completed;
-    int8_t method;
-
-} http_client;
-
-#ifdef SW_HAVE_ZLIB
-extern swString *swoole_zlib_buffer;
-#endif
+#include "swoole_http_client.h"
 
 static swString *http_client_buffer;
 
-static int http_client_parser_on_header_field(php_http_parser *parser, const char *at, size_t length);
-static int http_client_parser_on_header_value(php_http_parser *parser, const char *at, size_t length);
-static int http_client_parser_on_body(php_http_parser *parser, const char *at, size_t length);
-static int http_client_parser_on_headers_complete(php_http_parser *parser);
-static int http_client_parser_on_message_complete(php_http_parser *parser);
-
 static void http_client_onReceive(swClient *cli, char *data, uint32_t length);
 static void http_client_onConnect(swClient *cli);
 static void http_client_onClose(swClient *cli);
@@ -124,8 +29,6 @@ static void http_client_onResponseException();
 static int http_client_onMessage(swConnection *conn, char *data, uint32_t length);
 
 static int http_client_send_http_request(zval *zobject TSRMLS_DC);
-static http_client* http_client_create(zval *object TSRMLS_DC);
-static void http_client_free(zval *object TSRMLS_DC);
 static int http_client_execute(zval *zobject, char *uri, zend_size_t uri_len, zval *callback TSRMLS_DC);
 
 #ifdef SW_HAVE_ZLIB
@@ -135,51 +38,6 @@ extern voidpf php_zlib_alloc(voidpf opaque, uInt items, uInt size);
 extern void php_zlib_free(voidpf opaque, voidpf address);
 #endif
 
-static sw_inline void http_client_swString_append_headers(swString* swStr, char* key, zend_size_t key_len, char* data, zend_size_t data_len)
-{
-    swString_append_ptr(swStr, key, key_len);
-    swString_append_ptr(swStr, ZEND_STRL(": "));
-    swString_append_ptr(swStr, data, data_len);
-    swString_append_ptr(swStr, ZEND_STRL("\r\n"));
-}
-
-static sw_inline void http_client_append_content_length(swString* buf, int length)
-{
-    char content_length_str[32];
-    int n = snprintf(content_length_str, sizeof(content_length_str), "Content-Length: %d\r\n\r\n", length);
-    swString_append_ptr(buf, content_length_str, n);
-}
-
-static sw_inline void http_client_create_token(int length, char *buf)
-{
-    char characters[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!\"§$%&/()=[]{}";
-    int i;
-    assert(length < 1024);
-    for (i = 0; i < length; i++)
-    {
-        buf[i] = characters[rand() % sizeof(characters) - 1];
-    }
-    buf[length] = '\0';
-}
-
-static sw_inline int http_client_check_data(zval *data TSRMLS_DC)
-{
-    if (Z_TYPE_P(data) != IS_ARRAY && Z_TYPE_P(data) != IS_STRING)
-    {
-        swoole_php_error(E_WARNING, "parameter $data must be an array or string.");
-        return SW_ERR;
-    }
-    else if (Z_TYPE_P(data) == IS_ARRAY && php_swoole_array_length(data) == 0)
-    {
-        swoole_php_error(E_WARNING, "parameter $data is empty.");
-    }
-    else if (Z_TYPE_P(data) == IS_STRING && Z_STRLEN_P(data) == 0)
-    {
-        swoole_php_error(E_WARNING, "parameter $data is empty.");
-    }
-    return SW_OK;
-}
-
 static const php_http_parser_settings http_parser_settings =
 {
     NULL,
@@ -480,30 +338,11 @@ static int http_client_execute(zval *zobject, char *uri, zend_size_t uri_len, zv
                 return SW_ERR;
             }
             zval *value;
-            if (sw_zend_hash_find(Z_ARRVAL_P(send_header), ZEND_STRS("Host"), (void **) &value) == FAILURE ||
-                    Z_TYPE_P(value) != IS_STRING || Z_STRLEN_P(value) < 1)
+            if (sw_zend_hash_find(Z_ARRVAL_P(send_header), ZEND_STRS("Host"), (void **) &value) == FAILURE)
             {
-                swoole_php_fatal_error(E_WARNING, "http proxy must set Host");
+                swoole_php_fatal_error (E_WARNING, "http proxy must set Host");
                 return SW_ERR;
             }
-            if (http->cli->http_proxy->password)
-            {
-                char _buf1[128];
-                char _buf2[256];
-                int _n1 = snprintf(_buf1, sizeof(_buf1), "%*s:%*s", http->cli->http_proxy->l_user,
-                        http->cli->http_proxy->user, http->cli->http_proxy->l_password,
-                        http->cli->http_proxy->password);
-#if PHP_MAJOR_VERSION < 7
-                uchar *encoded_value = php_base64_encode((const unsigned char *) _buf1, _n1, NULL);
-                int _n2 = snprintf(_buf2, sizeof(_buf2), "Basic %*s", _n1, encoded_value);
-                add_assoc_stringl_ex(send_header, ZEND_STRS("Proxy-Authorization"), _buf2, _n2, 1);
-#else
-                zend_string *str = php_base64_encode((const unsigned char *) _buf1, _n1);
-                int _n2 = snprintf(_buf2, sizeof(_buf2), "Basic %*s", str->len, str->val);
-                zend_string_free(str);
-                add_assoc_stringl_ex(send_header, ZEND_STRL("Proxy-Authorization"), _buf2, _n2);
-#endif
-            }
         }
     }
 
@@ -860,21 +699,11 @@ static void http_client_onReceive(swClient *cli, char *data, uint32_t length)
     if (http->upgrade)
     {
         cli->open_length_check = 1;
+        swString_clear(cli->buffer);
         cli->protocol.get_package_length = swWebSocket_get_package_length;
         cli->protocol.onPackage = http_client_onMessage;
         cli->protocol.package_length_size = SW_WEBSOCKET_HEADER_LEN + SW_WEBSOCKET_MASK_LEN + sizeof(uint64_t);
         http->state = HTTP_CLIENT_STATE_UPGRADE;
-
-        //data frame
-        if (length > parsed_n)
-        {
-            cli->buffer->length = length - parsed_n - 1;
-            memmove(cli->buffer->str, data + parsed_n + 1, cli->buffer->length);
-        }
-        else
-        {
-            swString_clear(cli->buffer);
-        }
     }
     else if (http->keep_alive == 1)
     {
@@ -925,14 +754,6 @@ static void http_client_onReceive(swClient *cli, char *data, uint32_t length)
         return;
     }
     http->header_completed = 0;
-
-    if (http->upgrade && cli->buffer->length > 0)
-    {
-        cli->socket->skip_recv = 1;
-        swProtocol_recv_check_length(&cli->protocol, cli->socket, cli->buffer);
-        return;
-    }
-
     swString_clear(cli->buffer);
     if (http->keep_alive == 0 && http->state != HTTP_CLIENT_STATE_WAIT_CLOSE)
     {
@@ -1010,8 +831,6 @@ static int http_client_send_http_request(zval *zobject TSRMLS_DC)
         }
     }
 
-    http->method = swHttp_get_method(hcc->request_method, strlen(hcc->request_method) + 1);
-
     char *key;
     uint32_t keylen;
     int keytype;
@@ -1070,10 +889,6 @@ static int http_client_send_http_request(zval *zobject TSRMLS_DC)
                 continue;
             }
             convert_to_string(value);
-            if (Z_STRLEN_P(value) == 0)
-            {
-                continue;
-            }
             http_client_swString_append_headers(http_client_buffer, key, keylen, Z_STRVAL_P(value), Z_STRLEN_P(value));
         SW_HASHTABLE_FOREACH_END();
     }
@@ -1102,10 +917,6 @@ static int http_client_send_http_request(zval *zobject TSRMLS_DC)
                 continue;
             }
             convert_to_string(value);
-            if (Z_STRLEN_P(value) == 0)
-            {
-                continue;
-            }
             swString_append_ptr(http_client_buffer, key, keylen);
             swString_append_ptr(http_client_buffer, "=", 1);
 
@@ -1235,10 +1046,6 @@ static int http_client_send_http_request(zval *zobject TSRMLS_DC)
                 {
                     continue;
                 }
-                if (sw_zend_hash_find(Z_ARRVAL_P(value), ZEND_STRS("size"), (void **) &zsize) == FAILURE)
-                {
-                    continue;
-                }
                 if (sw_zend_hash_find(Z_ARRVAL_P(value), ZEND_STRS("type"), (void **) &ztype) == FAILURE)
                 {
                     continue;
@@ -1275,6 +1082,10 @@ static int http_client_send_http_request(zval *zobject TSRMLS_DC)
         {
             goto send_fail;
         }
+        else
+        {
+            return SW_OK;
+        }
     }
     //x-www-form-urlencoded or raw
     else if (post_data && !ZVAL_IS_NULL(post_data))
@@ -1283,28 +1094,25 @@ static int http_client_send_http_request(zval *zobject TSRMLS_DC)
         {
             zend_size_t len;
             http_client_swString_append_headers(http_client_buffer, ZEND_STRL("Content-Type"), ZEND_STRL("application/x-www-form-urlencoded"));
-            if (php_swoole_array_length(post_data) > 0) //if it's an empty array, http build will fail
+            smart_str formstr_s = { 0 };
+            char *formstr = sw_http_build_query(post_data, &len, &formstr_s TSRMLS_CC);
+            if (formstr == NULL)
             {
-                smart_str formstr_s = { 0 };
-                char *formstr = sw_http_build_query(post_data, &len, &formstr_s TSRMLS_CC);
-                if (formstr == NULL)
-                {
-                    swoole_php_error(E_WARNING, "http_build_query failed.");
-                    return SW_ERR;
-                }
-                http_client_append_content_length(http_client_buffer, len);
-                swString_append_ptr(http_client_buffer, formstr, len);
-                smart_str_free(&formstr_s);
+                swoole_php_error(E_WARNING, "http_build_query failed.");
+                return SW_ERR;
             }
-            else
-            {
-                http_client_append_content_length(http_client_buffer, 0);
-            }
-            //send http header and body
+            http_client_append_content_length(http_client_buffer, len);
+            //send http header
             if ((ret = http->cli->send(http->cli, http_client_buffer->str, http_client_buffer->length, 0)) < 0)
             {
                 goto send_fail;
             }
+            //send http body
+            if ((ret = http->cli->send(http->cli, formstr, len, 0)) < 0)
+            {
+                goto send_fail;
+            }
+            smart_str_free(&formstr_s);
             //cleanup request body
             zend_update_property_null(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("requestBody") TSRMLS_CC);
         }
@@ -1356,7 +1164,7 @@ static int http_client_send_http_request(zval *zobject TSRMLS_DC)
     return ret;
 }
 
-static void http_client_free(zval *object TSRMLS_DC)
+void http_client_free(zval *object TSRMLS_DC)
 {
     http_client *http = swoole_get_object(object);
     if (!http)
@@ -1390,9 +1198,11 @@ static void http_client_free(zval *object TSRMLS_DC)
         http->cli = NULL;
     }
     efree(http);
+
+    swTraceLog(SW_TRACE_HTTP_CLIENT, "free, object handle=%d.", sw_get_object_handle(object));
 }
 
-static http_client* http_client_create(zval *object TSRMLS_DC)
+http_client* http_client_create(zval *object TSRMLS_DC)
 {
     zval *ztmp;
     http_client *http;
@@ -1417,6 +1227,8 @@ static http_client* http_client_create(zval *object TSRMLS_DC)
     http->keep_alive = 1;
     http->state = HTTP_CLIENT_STATE_READY;
 
+    swTraceLog(SW_TRACE_HTTP_CLIENT, "create, object handle=%d.", sw_get_object_handle(object));
+
     return http;
 }
 
@@ -1510,10 +1322,6 @@ static PHP_METHOD(swoole_http_client, set)
     {
         return;
     }
-    if (Z_TYPE_P(zset) != IS_ARRAY)
-    {
-        RETURN_FALSE;
-    }
 
     zval *zsetting = php_swoole_read_init_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("setting") TSRMLS_CC);
     sw_php_array_merge(Z_ARRVAL_P(zsetting), Z_ARRVAL_P(zset));
@@ -1807,7 +1615,7 @@ static PHP_METHOD(swoole_http_client, on)
     RETURN_TRUE;
 }
 
-static int http_client_parser_on_header_field(php_http_parser *parser, const char *at, size_t length)
+int http_client_parser_on_header_field(php_http_parser *parser, const char *at, size_t length)
 {
     http_client* http = (http_client*) parser->data;
     http->tmp_header_field_name = (char *) at;
@@ -1815,7 +1623,7 @@ static int http_client_parser_on_header_field(php_http_parser *parser, const cha
     return 0;
 }
 
-static int http_client_parser_on_header_value(php_http_parser *parser, const char *at, size_t length)
+int http_client_parser_on_header_value(php_http_parser *parser, const char *at, size_t length)
 {
 #if PHP_MAJOR_VERSION < 7
     TSRMLS_FETCH_FROM_CTX(sw_thread_ctx ? sw_thread_ctx : NULL);
@@ -1995,7 +1803,7 @@ int http_response_uncompress(z_stream *stream, swString *buffer, char *body, int
 }
 #endif
 
-static int http_client_parser_on_body(php_http_parser *parser, const char *at, size_t length)
+int http_client_parser_on_body(php_http_parser *parser, const char *at, size_t length)
 {
     http_client* http = (http_client*) parser->data;
     if (swString_append_ptr(http->body, (char *) at, length) < 0)
@@ -2029,7 +1837,7 @@ static int http_client_parser_on_body(php_http_parser *parser, const char *at, s
     return 0;
 }
 
-static int http_client_parser_on_headers_complete(php_http_parser *parser)
+int http_client_parser_on_headers_complete(php_http_parser *parser)
 {
     http_client* http = (http_client*) parser->data;
     //no content-length
@@ -2037,14 +1845,10 @@ static int http_client_parser_on_headers_complete(php_http_parser *parser)
     {
         http->state = HTTP_CLIENT_STATE_WAIT_CLOSE;
     }
-    if (http->method == HTTP_HEAD)
-    {
-        return 1;
-    }
     return 0;
 }
 
-static int http_client_parser_on_message_complete(php_http_parser *parser)
+int http_client_parser_on_message_complete(php_http_parser *parser)
 {
 #if PHP_MAJOR_VERSION < 7
     TSRMLS_FETCH_FROM_CTX(sw_thread_ctx ? sw_thread_ctx : NULL);
@@ -2216,13 +2020,17 @@ static PHP_METHOD(swoole_http_client, upgrade)
     sw_add_assoc_string(headers, "Upgrade", "websocket", 1);
     sw_add_assoc_string(headers, "Sec-WebSocket-Version", SW_WEBSOCKET_VERSION, 1);
 
-#if PHP_MAJOR_VERSION < 7
     int encoded_value_len = 0;
+
+#if PHP_MAJOR_VERSION < 7
     uchar *encoded_value = php_base64_encode((const unsigned char *) buf, SW_WEBSOCKET_KEY_LENGTH, &encoded_value_len);
-    add_assoc_stringl_ex(headers, ZEND_STRS("Sec-WebSocket-Key"), (char* )encoded_value, encoded_value_len, 0);
+    add_assoc_stringl(headers, "Sec-WebSocket-Key", (char* )encoded_value, encoded_value_len, 0);
 #else
     zend_string *str = php_base64_encode((const unsigned char *) buf, SW_WEBSOCKET_KEY_LENGTH);
-    add_assoc_str_ex(headers, ZEND_STRL("Sec-WebSocket-Key"), str);
+    char *encoded_value = str->val;
+    encoded_value_len = str->len;
+    add_assoc_stringl(headers, "Sec-WebSocket-Key", (char* )encoded_value, encoded_value_len);
+    zend_string_free(str);
 #endif
 
     ret = http_client_execute(getThis(), uri, uri_len, finish_cb TSRMLS_CC);
