@@ -16,6 +16,9 @@
 
 #include "php_swoole.h"
 #include "swoole_http.h"
+#ifdef SW_COROUTINE
+#include "swoole_coroutine.h"
+#endif
 
 #include <ext/standard/url.h>
 #include <ext/standard/sha1.h>
@@ -104,6 +107,7 @@ static http_context* http_get_context(zval *object, int check_end TSRMLS_DC);
 
 static void http_parse_cookie(zval *array, const char *at, size_t length);
 static void http_build_header(http_context *, zval *object, swString *response, int body_length TSRMLS_DC);
+static int http_trim_double_quote(zval **value, char **ptr);
 
 static inline void http_header_key_format(char *key, int length)
 {
@@ -132,42 +136,6 @@ static inline void http_header_key_format(char *key, int length)
     }
 }
 
-static inline char* http_trim_double_quote(char *ptr, int *len)
-{
-    int i;
-    char *tmp = ptr;
-
-    //ltrim('"')
-    for (i = 0; i < *len; i++)
-    {
-        if (tmp[i] == '"')
-        {
-            (*len)--;
-            tmp++;
-            continue;
-        }
-        else
-        {
-            break;
-        }
-    }
-    //rtrim('"')
-    for (i = (*len) - 1; i >= 0; i--)
-    {
-        if (tmp[i] == '"')
-        {
-            tmp[i] = 0;
-            (*len)--;
-            continue;
-        }
-        else
-        {
-            break;
-        }
-    }
-    return tmp;
-}
-
 #ifdef SW_USE_PICOHTTPPARSER
 enum flags
 {
@@ -246,7 +214,6 @@ void php_zlib_free(voidpf opaque, voidpf address);
 static PHP_METHOD(swoole_http_server, on);
 static PHP_METHOD(swoole_http_server, start);
 
-static PHP_METHOD(swoole_http_request, getData);
 static PHP_METHOD(swoole_http_request, rawcontent);
 static PHP_METHOD(swoole_http_request, __destruct);
 
@@ -418,7 +385,6 @@ const zend_function_entry swoole_http_server_methods[] =
 const zend_function_entry swoole_http_request_methods[] =
 {
     PHP_ME(swoole_http_request, rawcontent, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_http_request, getData, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
     PHP_FALIAS(__sleep, swoole_unsupport_serialize, NULL)
     PHP_FALIAS(__wakeup, swoole_unsupport_serialize, NULL)
     PHP_ME(swoole_http_request, __destruct, arginfo_swoole_http_void, ZEND_ACC_PUBLIC | ZEND_ACC_DTOR)
@@ -510,7 +476,6 @@ static void http_parse_cookie(zval *array, const char *at, size_t length)
     char valbuf[SW_HTTP_COOKIE_VALLEN];
     char *_c = (char *) at;
 
-    char *_value;
     int klen = 0;
     int vlen = 0;
     int state = -1;
@@ -542,11 +507,10 @@ static void http_parse_cookie(zval *array, const char *at, size_t length)
             }
             memcpy(valbuf, (char *) at + j, vlen);
             valbuf[vlen] = 0;
-            _value = http_trim_double_quote(valbuf, &vlen);
-            vlen = php_url_decode(_value, vlen);
+            vlen = php_url_decode(valbuf, vlen);
             if (klen > 1)
             {
-                sw_add_assoc_stringl_ex(array, keybuf, klen, _value, vlen, 1);
+                sw_add_assoc_stringl_ex(array, keybuf, klen, valbuf, vlen, 1);
             }
             j = i + 1;
             state = -1;
@@ -581,16 +545,34 @@ static void http_parse_cookie(zval *array, const char *at, size_t length)
             return;
         }
         memcpy(valbuf, (char *) at + j, vlen);
-        valbuf[vlen] = 0;
-        _value = http_trim_double_quote(valbuf, &vlen);
-        vlen = php_url_decode(_value, vlen);
+        valbuf[vlen] = 0;;
+        vlen = php_url_decode(valbuf, vlen);
         if (klen > 1)
         {
-            sw_add_assoc_stringl_ex(array, keybuf, klen, _value, vlen, 1);
+            sw_add_assoc_stringl_ex(array, keybuf, klen, valbuf, vlen, 1);
         }
     }
 }
 
+static int http_trim_double_quote(zval **value, char **ptr)
+{
+    int len = Z_STRLEN_PP(value);
+    *ptr = Z_STRVAL_PP(value);
+
+    //ltrim('"')
+    if ((*ptr)[0] == '"')
+    {
+        (*ptr)++;
+        len--;
+    }
+    //rtrim('"')
+    if ((*ptr)[len - 1] == '"')
+    {
+        len--;
+    }
+    return len;
+}
+
 static int http_request_on_header_value(php_http_parser *parser, const char *at, size_t length)
 {
 #if PHP_MAJOR_VERSION < 7
@@ -639,28 +621,21 @@ static int http_request_on_header_value(php_http_parser *parser, const char *at,
             {
                 offset = sizeof("multipart/form-data;") - 1;
 
-                while (at[offset] == ' ')
-                {
+                while (at[offset] == ' ') {
                     offset += 1;
                 }
 
                 offset += sizeof("boundary=") - 1;
 
                 int boundary_len = length - offset;
-                char *boundary_str = (char *) at + length - boundary_len;
 
                 if (boundary_len <= 0)
                 {
                     swWarn("invalid multipart/form-data body.", ctx->fd);
                     return 0;
                 }
-                if (boundary_len >= 2 && boundary_str[0] == '"' && *(boundary_str + boundary_len - 1) == '"')
-                {
-                    boundary_str++;
-                    boundary_len -= 2;
-                }
 
-                swoole_http_parse_form_data(ctx, boundary_str, boundary_len TSRMLS_CC);
+                swoole_http_parse_form_data(ctx, at + length - boundary_len, boundary_len TSRMLS_CC);
             }
         }
     }
@@ -704,9 +679,6 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
     TSRMLS_FETCH_FROM_CTX(sw_thread_ctx ? sw_thread_ctx : NULL);
 #endif
 
-    char value_buf[SW_HTTP_COOKIE_KEYLEN];
-    int value_len;
-
     http_context *ctx = p->data;
     /**
      * Hash collision attack
@@ -744,27 +716,20 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
             return SW_OK;
         }
 
-        if (Z_STRLEN_P(form_name) >= SW_HTTP_COOKIE_KEYLEN)
-        {
-            swWarn("form_name[%s] is too large.", Z_STRVAL_P(form_name));
-            return SW_OK;
-        }
-
-        strncpy(value_buf, Z_STRVAL_P(form_name), Z_STRLEN_P(form_name));
-        value_len = Z_STRLEN_P(form_name);
-        char *tmp = http_trim_double_quote(value_buf, &value_len);
+        char *str;
+        int len = http_trim_double_quote(&form_name, &str);
 
         zval *filename;
         //POST form data
         if (sw_zend_hash_find(Z_ARRVAL_P(tmp_array), ZEND_STRS("filename"), (void **) &filename) == FAILURE)
         {
-            ctx->current_form_data_name = estrndup(tmp, value_len);
-            ctx->current_form_data_name_len = value_len;
+            ctx->current_form_data_name = estrndup(str, len);
+            ctx->current_form_data_name_len = len;
         }
         //upload file
         else
         {
-            ctx->current_input_name = estrndup(tmp, value_len);
+            ctx->current_input_name = estrndup(str, len);
 
             zval *multipart_header = NULL;
             SW_ALLOC_INIT_ZVAL(multipart_header);
@@ -776,11 +741,8 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
             add_assoc_long(multipart_header, "error", HTTP_UPLOAD_ERR_OK);
             add_assoc_long(multipart_header, "size", 0);
 
-            strncpy(value_buf, Z_STRVAL_P(filename), Z_STRLEN_P(filename));
-            value_len = Z_STRLEN_P(filename);
-            tmp = http_trim_double_quote(value_buf, &value_len);
-
-            sw_add_assoc_stringl(multipart_header, "name", tmp, value_len, 1);
+            len = http_trim_double_quote(&filename, &str);
+            sw_add_assoc_stringl(multipart_header, "name", str, len, 1);
 
             ctx->current_multipart_header = multipart_header;
         }
@@ -979,12 +941,7 @@ static int http_request_on_body(php_http_parser *parser, const char *at, size_t
     else if (ctx->mt_parser != NULL)
     {
         multipart_parser *multipart_parser = ctx->mt_parser;
-        char *c = (char *) at;
-        while (*c == '\r' && *(c + 1) == '\n')
-        {
-            c += 2;
-        }
-        size_t n = multipart_parser_execute(multipart_parser, c, length);
+        size_t n = multipart_parser_execute(multipart_parser, at, length);
         if (n != length)
         {
             swoole_php_fatal_error(E_WARNING, "parse multipart body failed.");
@@ -1101,7 +1058,11 @@ static int http_onReceive(swServer *serv, swEventData *req)
     else
     {
         zval *retval = NULL;
+#ifndef SW_COROUTINE
         zval **args[2];
+#else
+        zval *args[2];
+#endif
 
         zval *zrequest_object = ctx->request.zobject;
         zval *zresponse_object = ctx->response.zobject;
@@ -1134,6 +1095,7 @@ static int http_onReceive(swServer *serv, swEventData *req)
         add_assoc_long(ctx->request.zserver, "server_port", swConnection_get_port(&SwooleG.serv->connection_list[conn->from_fd]));
         add_assoc_long(ctx->request.zserver, "remote_port", swConnection_get_port(conn));
         sw_add_assoc_string(zserver, "remote_addr", swConnection_get_ip(conn), 1);
+        add_assoc_long(zserver, "master_time", conn->last_time);
 
         if (ctx->request.version == 101)
         {
@@ -1154,8 +1116,13 @@ static int http_onReceive(swServer *serv, swEventData *req)
             goto free_object;
         }
 
+#ifndef SW_COROUTINE
         args[0] = &zrequest_object;
         args[1] = &zresponse_object;
+#else
+        args[0] = zrequest_object;
+        args[1] = zresponse_object;
+#endif
 
         int callback_type = 0;
         if (conn->websocket_status == WEBSOCKET_STATUS_CONNECTION)
@@ -1176,17 +1143,31 @@ static int http_onReceive(swServer *serv, swEventData *req)
             }
         }
 
+#ifndef SW_COROUTINE
         zcallback = php_swoole_server_get_callback(serv, req->info.from_fd, callback_type);
 #ifdef PHP_SWOOLE_ENABLE_FASTCALL
         zend_fcall_info_cache *fci_cache = php_swoole_server_get_cache(serv, req->info.from_fd, callback_type);
         if (sw_call_user_function_fast(zcallback, fci_cache, &retval, 2, args TSRMLS_CC) == FAILURE)
 #else
-
         if (sw_call_user_function_ex(EG(function_table), NULL, zcallback, &retval, 2, args, 0, NULL TSRMLS_CC) == FAILURE)
 #endif
         {
             swoole_php_error(E_WARNING, "onRequest handler error");
         }
+#else
+        zend_fcall_info_cache *cache = php_swoole_server_get_cache(serv, req->info.from_fd, callback_type);
+        int ret = coro_create(cache, args, 2, &retval, NULL, NULL);
+        if (ret != 0)
+        {
+            sw_zval_ptr_dtor(&zrequest_object);
+            sw_zval_ptr_dtor(&zresponse_object);
+            if (ret == CORO_LIMIT)
+            {
+                SwooleG.serv->factory.end(&SwooleG.serv->factory, fd);
+            }
+            return SW_OK;
+        }
+#endif
         if (EG(exception))
         {
             zend_exception_error(EG(exception), E_ERROR TSRMLS_CC);
@@ -1289,10 +1270,14 @@ static PHP_METHOD(swoole_http_server, on)
         return;
     }
 
-#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+#ifdef PHP_SWOOLE_CHECK_CALLBACK
     char *func_name = NULL;
+#if !defined(SW_COROUTINE) && !defined(PHP_SWOOLE_ENABLE_FASTCALL)
+    if (!sw_zend_is_callable(callback, 0, &func_name TSRMLS_CC))
+#else
     zend_fcall_info_cache *func_cache = emalloc(sizeof(zend_fcall_info_cache));
     if (!sw_zend_is_callable_ex(callback, NULL, 0, &func_name, NULL, func_cache, NULL TSRMLS_CC))
+#endif
     {
         swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
         efree(func_name);
@@ -1315,7 +1300,7 @@ static PHP_METHOD(swoole_http_server, on)
         zend_update_property(swoole_http_server_class_entry_ptr, getThis(), ZEND_STRL("onRequest"), callback TSRMLS_CC);
         php_sw_server_callbacks[SW_SERVER_CB_onRequest] = sw_zend_read_property(swoole_http_server_class_entry_ptr, getThis(), ZEND_STRL("onRequest"), 0 TSRMLS_CC);
         sw_copy_to_stack(php_sw_server_callbacks[SW_SERVER_CB_onRequest], _php_sw_server_callbacks[SW_SERVER_CB_onRequest]);
-#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+#if defined(SW_COROUTINE) || defined(PHP_SWOOLE_ENABLE_FASTCALL)
         php_sw_server_caches[SW_SERVER_CB_onRequest] = func_cache;
 #endif
     }
@@ -1324,13 +1309,13 @@ static PHP_METHOD(swoole_http_server, on)
         zend_update_property(swoole_http_server_class_entry_ptr, getThis(), ZEND_STRL("onHandshake"), callback TSRMLS_CC);
         php_sw_server_callbacks[SW_SERVER_CB_onHandShake] = sw_zend_read_property(swoole_http_server_class_entry_ptr, getThis(), ZEND_STRL("onHandshake"), 0 TSRMLS_CC);
         sw_copy_to_stack(php_sw_server_callbacks[SW_SERVER_CB_onHandShake], _php_sw_server_callbacks[SW_SERVER_CB_onHandShake]);
-#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+#if defined(SW_COROUTINE) || defined(PHP_SWOOLE_ENABLE_FASTCALL)
         php_sw_server_caches[SW_SERVER_CB_onHandShake] = func_cache;
 #endif
     }
     else
     {
-#ifdef PHP_SWOOLE_ENABLE_FASTCALL
+#if defined(SW_COROUTINE) || defined(PHP_SWOOLE_ENABLE_FASTCALL)
         efree(func_cache);
 #endif
         zval *obj = getThis();
@@ -1601,9 +1586,6 @@ static PHP_METHOD(swoole_http_server, start)
     {
         SW_ALLOC_INIT_ZVAL(zsetting);
         array_init(zsetting);
-#ifdef HT_ALLOW_COW_VIOLATION
-        HT_ALLOW_COW_VIOLATION(Z_ARRVAL_P(zsetting));
-#endif
         zend_update_property(swoole_server_class_entry_ptr, getThis(), ZEND_STRL("setting"), zsetting TSRMLS_CC);
     }
 
@@ -1622,6 +1604,8 @@ static PHP_METHOD(swoole_http_server, start)
     serv->listen_list->open_eof_check = 0;
     serv->listen_list->open_length_check = 0;
 
+    serv->ptr2 = getThis();
+
     //for is_uploaded_file and move_uploaded_file
     ALLOC_HASHTABLE(SG(rfc1867_uploaded_files));
     zend_hash_init(SG(rfc1867_uploaded_files), 8, NULL, NULL, 0);
@@ -1639,6 +1623,13 @@ static PHP_METHOD(swoole_http_server, start)
 
 static PHP_METHOD(swoole_http_request, rawcontent)
 {
+    zval *zfd = sw_zend_read_property(swoole_http_request_class_entry_ptr, getThis(), ZEND_STRL("fd"), 0 TSRMLS_CC);
+    if (ZVAL_IS_NULL(zfd))
+    {
+        swoole_php_error(E_WARNING, "http client is not existed.");
+        RETURN_FALSE;
+    }
+
     http_context *ctx = http_get_context(getThis(), 0 TSRMLS_CC);
     if (!ctx)
     {
@@ -1663,19 +1654,6 @@ static PHP_METHOD(swoole_http_request, rawcontent)
     }
 }
 
-static PHP_METHOD(swoole_http_request, getData)
-{
-    zval *zdata = swoole_get_property(getThis(), 0);
-    if (zdata)
-    {
-        SW_RETURN_STRINGL(Z_STRVAL_P(zdata), Z_STRLEN_P(zdata), 1);
-    }
-    else
-    {
-        RETURN_FALSE;
-    }
-}
-
 static PHP_METHOD(swoole_http_request, __destruct)
 {
     zval *ztmpfiles = sw_zend_read_property(swoole_http_request_class_entry_ptr, getThis(), ZEND_STRL("tmpfiles"), 1 TSRMLS_CC);
@@ -2143,16 +2121,6 @@ static PHP_METHOD(swoole_http_response, end)
             RETURN_FALSE;
         }
     }
-
-    if (ctx->upgrade)
-    {
-        swConnection *conn = swWorker_get_connection(SwooleG.serv, ctx->fd);
-        if (conn && conn->websocket_status == WEBSOCKET_STATUS_HANDSHAKE && ctx->response.status == 101)
-        {
-            conn->websocket_status = WEBSOCKET_STATUS_ACTIVE;
-        }
-    }
-
     if (!ctx->keepalive)
     {
         swServer_tcp_close(SwooleG.serv, ctx->fd, 0);
@@ -2473,13 +2441,13 @@ static PHP_METHOD(swoole_http_response, status)
         return;
     }
 
-    http_context *ctx = http_get_context(getThis(), 0 TSRMLS_CC);
-    if (!ctx)
+    http_context *client = http_get_context(getThis(), 0 TSRMLS_CC);
+    if (!client)
     {
         RETURN_FALSE;
     }
 
-    ctx->response.status = http_status;
+    client->response.status = http_status;
 }
 
 static PHP_METHOD(swoole_http_response, header)
@@ -2642,8 +2610,6 @@ static PHP_METHOD(swoole_http_response, __destruct)
         }
         else
         {
-            context->response.status = 500;
-
             zval *zobject = getThis();
             zval *retval = NULL;
             sw_zend_call_method_with_0_params(&zobject, swoole_http_response_class_entry_ptr, NULL, "end", &retval);
