@@ -122,7 +122,6 @@ enum mysql_field_types
     SW_MYSQL_TYPE_NEWDATE,
     SW_MYSQL_TYPE_VARCHAR,
     SW_MYSQL_TYPE_BIT,
-    SW_MYSQL_TYPE_JSON = 245,
     SW_MYSQL_TYPE_NEWDECIMAL = 246,
     SW_MYSQL_TYPE_ENUM = 247,
     SW_MYSQL_TYPE_SET = 248,
@@ -135,6 +134,16 @@ enum mysql_field_types
     SW_MYSQL_TYPE_GEOMETRY = 255
 };
 
+#ifdef SW_COROUTINE
+typedef enum
+{
+	SW_MYSQL_CORO_STATUS_CLOSED,
+	SW_MYSQL_CORO_STATUS_READY,
+	SW_MYSQL_CORO_STATUS_WAIT,
+	SW_MYSQL_CORO_STATUS_DONE
+} mysql_io_status;
+#endif
+
 #define SW_MYSQL_CLIENT_CONNECT_WITH_DB          8
 #define SW_MYSQL_CLIENT_PROTOCOL_41              512
 #define SW_MYSQL_CLIENT_PLUGIN_AUTH              (1UL << 19)
@@ -249,6 +258,13 @@ typedef struct
 
 typedef struct
 {
+#ifdef SW_COROUTINE
+    zend_bool defer;
+    zend_bool _defer;
+    mysql_io_status iowait;
+    zval *result;
+    int cid;
+#endif
     uint8_t state;
     uint8_t handshake;
     swString *buffer;
@@ -288,6 +304,17 @@ typedef struct
                                     (((uint32_t) ((zend_uchar) (A)[6])) << 16) +\
                                     (((uint32_t) ((zend_uchar) (A)[7])) << 24))) << 32))
 
+int mysql_get_result(mysql_connector *connector, char *buf, int len);
+int mysql_get_charset(char *name);
+int mysql_handshake(mysql_connector *connector, char *buf, int len);
+int mysql_request(swString *sql, swString *buffer);
+int mysql_response(mysql_client *client);
+
+#ifdef SW_MYSQL_DEBUG
+void mysql_client_info(mysql_client *client);
+void mysql_column_info(mysql_field *field);
+#endif
+
 static sw_inline void mysql_pack_length(int length, char *buf)
 {
     buf[2] = length >> 16;
@@ -605,7 +632,6 @@ static sw_inline int mysql_decode_row(mysql_client *client, char *buf, int packe
         case SW_MYSQL_TYPE_DECIMAL:
         case SW_MYSQL_TYPE_NEWDECIMAL:
         case SW_MYSQL_TYPE_BIT:
-        case SW_MYSQL_TYPE_JSON:
         case SW_MYSQL_TYPE_STRING:
         case SW_MYSQL_TYPE_VAR_STRING:
         case SW_MYSQL_TYPE_VARCHAR:
